"""
    Browser type detection
    ======= ==== =========

Detect the users browser type and version (Based on an internal list)
from the value of the request variable HTTP_USER_AGENT.

If you add a browser to this list, please send me your additions so
that I can include them in the next release. You will also get a credit
in this DocString!!!

Browser                 Contributed by
Konqueror/2.0           Neil Padgen [nrp@burla.force9.co.uk]
Konqueror/2.1           Neil Padgen [nrp@burla.force9.co.uk]
"""
# $Id: BrowserType.py,v 1.3 2002/01/24 00:27:15 adrian Exp $
#
# $Log: BrowserType.py,v $
# Revision 1.3  2002/01/24 00:27:15  adrian
# Fixed minor bug that allowed the browser lists to get out of sync with the
# setting of the useBrowserVersions attribute.
# Removed useBrowserVersions attribute from ZSSAttribute
#
# Revision 1.2  2001/10/21 14:48:33  adrian
# no message
#
#
__version__ = '$Revision: 1.3 $'[11:-2]

import string
import Acquisition

_browserTag   = [
        {'Text': 'Opera/2', 'Exclude': '', 'Name': 'Opera', 'Version': 2.0},
        {'Text': 'Opera/3', 'Exclude': '', 'Name': 'Opera', 'Version': 3.0},
        {'Text': 'Opera/4', 'Exclude': '', 'Name': 'Opera', 'Version': 4.0},
        {'Text': 'Opera/5', 'Exclude': '', 'Name': 'Opera', 'Version': 5.0},
        {'Text': 'MSIE 3',  'Exclude': '', 'Name': 'MSIE', 'Version': 3.0},
        {'Text': 'MSIE 4', 'Exclude': '', 'Name': 'MSIE', 'Version': 4.0},
        {'Text': 'MSIE 5', 'Exclude': '', 'Name': 'MSIE', 'Version': 5.0},
        {'Text': 'MSIE 5.5', 'Exclude': '', 'Name': 'MSIE', 'Version': 5.5},
        {'Text': 'MSIE 6.0', 'Exclude': '', 'Name': 'MSIE', 'Version': 6.0},
        {'Text': 'Mozilla/2', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 2.0},
        {'Text': 'Mozilla/3', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 3.0},
        {'Text': 'Mozilla/4', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 4.0},
        {'Text': 'Mozilla/5', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 5.0},
        {'Text': 'Mozilla/6', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 6.0},
        {'Text': 'Konqueror/2.0', 'Exclude': '', 'Name': 'Konqueror', 'Version': 2.0},
        {'Text': 'Konqueror/2.1', 'Exclude': '', 'Name': 'Konqueror', 'Version': 2.1},
        {'Text': 'Lynx/2.7', 'Exclude': '', 'Name': 'Lynx', 'Version': 2.7},
        {'Text': 'Lynx/2.8', 'Exclude': '', 'Name': 'Lynx', 'Version': 2.8},
    ]

class BrowserType:
    """
        Browser type detection mix-in class
    """
    useBrowserVersions = 0

    def _splitBrowser(self):
        """
            Split the REQUEST HEADER (HTTP_USER_AGENT)
            to crunch out the name and version
        """
        ua="%s" % self.REQUEST["HTTP_USER_AGENT"]
        Name=""
        Version=0
        for ThisBrowser in _browserTag:
            if (string.find(ua, ThisBrowser["Text"])!=-1):
                if ((string.find(ua, ThisBrowser["Exclude"])==-1) or
                    (not ThisBrowser["Exclude"])):
                    Name="%s" % ThisBrowser["Name"]
                    Version=ThisBrowser["Version"] * 1.0
        if (Name==""):
            Name="Other"
            Version=1.0
        return (Name, Version)

    def _testBrowser(self, browserList):
        """
            Is the browser in the list?
            (Your name's not on the list, You're not getting in...)
        """
        if type(browserList) != type([]):
            browserList = [browserList]
        if self.useBrowserVersions:
            if "%s (%1.1f)" % self._splitBrowser() in browserList:
                return 1
        else:
            if self._splitBrowser()[0] in browserList:
                return 1
        return 0

    def BrowserList(self):
        """
            Return a list of the known browsers plus the catch all 'Other'
        """
        # Ugly... need to make this give me the CORRECT list
        useBrowserVersions = Acquisition.aq_acquire(self, 'useBrowserVersions')
        print str(useBrowserVersions)
        List = []
        for ThisBrowser in _browserTag:
            if useBrowserVersions:
                ThisName = "%s (%1.1f)" % (ThisBrowser["Name"], ThisBrowser['Version'])
            else:
                ThisName = "%s" % ThisBrowser["Name"]
            if not(ThisName in List):
                List.append(ThisName)
        List.append("Other")
        return List

    def BrowserCount(self):
        """
            Return the number of items in the BrowserList
        """
        return len(self.BrowserList())

# These methods are not used directly by ZStyleSheet, but you can
# use them yourself to detect your browser type and version.

    def BrowserName(self):
        """
            Return the NAME of the browser type
        """
        (Name, Version) = self._splitBrowser()
        return Name

    def BrowserVersion(self):
        """
            Return the VERSION of the browser
        """
        (Name, Version) = self._splitBrowser()
        return Version

    def isBrowser(self, BrowserName):
        """
            Returns 'Browser Version' if this is the
            named browser, 0 otherwise (Float)
        """
        (Name, Version) = self._splitBrowser()
        if (Name == BrowserName):
            return Version
        return 0
